@@ -1,6 +1,7 @@
 import numpy as np
-from . import lib
-from .utils import _CArray
+
+from . import lib, utils
+from .utils import _CArray, output_type
 
 
 class _AllIndices:
@@ -41,13 +42,13 @@ class IndexerResolver:
                 raise TypeError(f"Index for {type(self.obj).__name__} cannot be a tuple")
             # Convert to tuple for consistent processing
             indices = (indices,)
-        elif len(shape) == 2:
+        else:  # len(shape) == 2
             if type(indices) is not tuple or len(indices) != 2:
                 raise TypeError(f"Index for {type(self.obj).__name__} must be a 2-tuple")
 
         out = []
         for i, idx in enumerate(indices):
-            typ = type(idx)
+            typ = output_type(idx)
             if typ is tuple:
                 raise TypeError(
                     f"Index in position {i} cannot be a tuple; must use slice or list or int"
@@ -60,7 +61,7 @@ class IndexerResolver:
 
         if np.issubdtype(typ, np.integer):
             if index >= size:
-                raise IndexError(f"index={index}, size={size}")
+                raise IndexError(f"Index out of range: index={index}, size={size}")
             return _CScalar(int(index)), None
         if typ is list:
             pass
@@ -83,8 +84,8 @@ class IndexerResolver:
                     raise TypeError(f"An integer is required for indexing.  Got: {index.dtype}")
                 return _CScalar(index), None
 
-            from .vector import Vector
             from .matrix import Matrix, TransposedMatrix
+            from .vector import Vector
 
             if typ is Vector or typ is Matrix:
                 raise TypeError(
@@ -333,17 +334,35 @@ class Updater:
 
 
 class InfixExprBase:
-    __slots__ = "left", "right", "__weakref__"
+    __slots__ = "left", "right", "_value", "__weakref__"
+    _is_scalar = False
 
     def __init__(self, left, right):
         self.left = left
         self.right = right
+        self._value = None
 
     def new(self, *, dtype=None, mask=None, name=None):
+        if (
+            mask is None
+            and self._value is not None
+            and (dtype is None or self._value.dtype == dtype)
+        ):
+            rv = self._value
+            if name is not None:
+                rv.name = name
+            self._value = None
+            return rv
         # Rely on the default operator for the method
         expr = getattr(self.left, self.method_name)(self.right)
         return expr.new(dtype=dtype, mask=mask, name=name)
 
+    dup = new
+
+    def _to_expr(self):
+        # Rely on the default operator for the method
+        return getattr(self.left, self.method_name)(self.right)
+
     def _format_expr(self):
         return f"{self.left.name} {self._infix} {self.right.name}"
 
@@ -368,228 +387,14 @@ class InfixExprBase:
             return formatting.format_matrix_infix_expression(self)
         return formatting.format_scalar_infix_expression(self)
 
-
-class VectorEwiseAddExpr(InfixExprBase):
-    __slots__ = "_size"
-    method_name = "ewise_add"
-    output_type = None  # VectorExpression
-    _infix = "|"
-    _example_op = "plus"
-
-    def __init__(self, left, right):
-        super().__init__(left, right)
-        self._size = left.size
-
     @property
-    def size(self):
-        return self._size
+    def dtype(self):
+        if self._value is not None:
+            return self._value.dtype
+        return self._to_expr().dtype
 
 
-class VectorEwiseMultExpr(InfixExprBase):
-    __slots__ = "_size"
-    method_name = "ewise_mult"
-    output_type = None  # VectorExpression
-    _infix = "&"
-    _example_op = "times"
-
-    def __init__(self, left, right):
-        super().__init__(left, right)
-        self._size = left.size
-
-    @property
-    def size(self):
-        return self._size
-
-
-class VectorMatMulExpr(InfixExprBase):
-    __slots__ = "method_name", "_size"
-    output_type = None  # VectorExpression
-    _infix = "@"
-    _example_op = "plus_times"
-
-    def __init__(self, left, right, *, method_name, size):
-        super().__init__(left, right)
-        self.method_name = method_name
-        self._size = size
-
-    @property
-    def size(self):
-        return self._size
-
-
-class MatrixEwiseAddExpr(InfixExprBase):
-    __slots__ = "_nrows", "_ncols"
-    method_name = "ewise_add"
-    output_type = None  # MatrixExpression
-    _infix = "|"
-    _example_op = "plus"
-
-    def __init__(self, left, right):
-        super().__init__(left, right)
-        self._nrows = left.nrows
-        self._ncols = left.ncols
-
-    @property
-    def nrows(self):
-        return self._nrows
-
-    @property
-    def ncols(self):
-        return self._ncols
-
-
-class MatrixEwiseMultExpr(InfixExprBase):
-    __slots__ = "_nrows", "_ncols"
-    method_name = "ewise_mult"
-    output_type = None  # MatrixExpression
-    _infix = "&"
-    _example_op = "times"
-
-    def __init__(self, left, right):
-        super().__init__(left, right)
-        self._nrows = left.nrows
-        self._ncols = left.ncols
-
-    @property
-    def nrows(self):
-        return self._nrows
-
-    @property
-    def ncols(self):
-        return self._ncols
-
-
-class MatrixMatMulExpr(InfixExprBase):
-    __slots__ = "_nrows", "_ncols"
-    method_name = "mxm"
-    output_type = None  # MatrixExpression
-    _infix = "@"
-    _example_op = "plus_times"
-
-    def __init__(self, left, right, *, nrows, ncols):
-        super().__init__(left, right)
-        self._nrows = nrows
-        self._ncols = ncols
-
-    @property
-    def nrows(self):
-        return self._nrows
-
-    @property
-    def ncols(self):
-        return self._ncols
-
-
-class ScalarMatMulExpr(InfixExprBase):
-    __slots__ = ()
-    method_name = "inner"
-    output_type = None  # ScalarExpression
-    _infix = "@"
-    _example_op = "plus_times"
-
-    def new(self, *, dtype=None, name=None):
-        # Rely on the default operator for the method
-        expr = getattr(self.left, self.method_name)(self.right)
-        return expr.new(dtype=dtype, name=name)
-
-    @property
-    def value(self):
-        return self.new().value
-
-
-def _ewise_infix_expr(left, right, *, method, within):
-    from .vector import Vector
-    from .matrix import Matrix, TransposedMatrix
-
-    left_type = type(left)
-    right_type = type(right)
-
-    if left_type in {Vector, Matrix, TransposedMatrix}:
-        if not (
-            left_type is right_type
-            or (left_type is Matrix and right_type is TransposedMatrix)
-            or (left_type is TransposedMatrix and right_type is Matrix)
-        ):
-            if left_type is Vector:
-                left._expect_type(right, Vector, within=within, argname="right")
-            else:
-                left._expect_type(right, (Matrix, TransposedMatrix), within=within, argname="right")
-    elif right_type is Vector:
-        right._expect_type(left, Vector, within=within, argname="left")
-    elif right_type is Matrix or right_type is TransposedMatrix:
-        right._expect_type(left, (Matrix, TransposedMatrix), within=within, argname="left")
-    else:  # pragma: no cover
-        raise TypeError(f"Bad types for ewise infix: {type(left).__name__}, {type(right).__name__}")
-
-    from .binary import any
-
-    # Create dummy expression to check compatibility of dimensions, etc.
-    expr = getattr(left, method)(right, any)
-    if expr.output_type is Vector:
-        if method == "ewise_mult":
-            return VectorEwiseMultExpr(left, right)
-        return VectorEwiseAddExpr(left, right)
-    elif method == "ewise_mult":
-        return MatrixEwiseMultExpr(left, right)
-    return MatrixEwiseAddExpr(left, right)
-
-
-def _matmul_infix_expr(left, right, *, within):
-    from .vector import Vector
-    from .matrix import Matrix, TransposedMatrix
-
-    left_type = type(left)
-    right_type = type(right)
-
-    if left_type is Vector:
-        if right_type is Matrix or right_type is TransposedMatrix:
-            method = "vxm"
-        elif right_type is Vector:
-            method = "inner"
-        else:
-            left._expect_type(
-                right,
-                (Matrix, TransposedMatrix),
-                within=within,
-                argname="right",
-            )
-    elif left_type is Matrix or left_type is TransposedMatrix:
-        if right_type is Vector:
-            method = "mxv"
-        elif right_type is Matrix or right_type is TransposedMatrix:
-            method = "mxm"
-        else:
-            left._expect_type(
-                right,
-                (Vector, Matrix, TransposedMatrix),
-                within=within,
-                argname="right",
-            )
-    elif right_type is Vector:
-        right._expect_type(
-            left,
-            (Matrix, TransposedMatrix),
-            within=within,
-            argname="left",
-        )
-    elif right_type is Matrix or right_type is TransposedMatrix:
-        right._expect_type(
-            left,
-            (Vector, Matrix, TransposedMatrix),
-            within=within,
-            argname="left",
-        )
-    else:  # pragma: no cover
-        raise TypeError(
-            f"Bad types for matmul infix: {type(left).__name__}, {type(right).__name__}"
-        )
-
-    from .semiring import any_pair
-
-    # Create dummy expression to check compatibility of dimensions, etc.
-    expr = getattr(left, method)(right, any_pair[bool])
-    if expr.output_type is Vector:
-        return VectorMatMulExpr(left, right, method_name=method, size=expr._size)
-    elif expr.output_type is Matrix:
-        return MatrixMatMulExpr(left, right, nrows=expr.nrows, ncols=expr.ncols)
-    return ScalarMatMulExpr(left, right)
+# Mistakes
+utils._output_types[AmbiguousAssignOrExtract] = AmbiguousAssignOrExtract
+utils._output_types[Assigner] = Assigner
+utils._output_types[Updater] = Updater
