@@ -135,7 +135,7 @@ class Matrix(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce_scalar(monoid.land).value
+        return matches.reduce_scalar(monoid.land).new().value
 
     def isclose(self, other, *, rel_tol=1e-7, abs_tol=0.0, check_dtype=False):
         """
@@ -163,7 +163,7 @@ class Matrix(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce_scalar(monoid.land).value
+        return matches.reduce_scalar(monoid.land).new().value
 
     @property
     def nrows(self):
@@ -253,9 +253,9 @@ class Matrix(BaseType):
             self.clear()
         if nrows is not None or ncols is not None:
             if nrows is None:
-                nrows = self.nrows
+                nrows = self._nrows
             if ncols is None:
-                ncols = self.ncols
+                ncols = self._ncols
             self.resize(nrows, ncols)
         if n == 0:
             return
@@ -777,7 +777,11 @@ class Matrix(BaseType):
         value_type = output_type(value)
         if value_type is Vector:
             if type(value) is not Vector:
-                value = value._get_value()
+                value = self._expect_type(
+                    value,
+                    Vector,
+                    within=method_name,
+                )
             if rowsize is None and colsize is not None:
                 # Row-only selection
                 row_index = rows
@@ -892,7 +896,11 @@ class Matrix(BaseType):
                 )
         elif value_type in {Matrix, TransposedMatrix}:
             if type(value) not in {Matrix, TransposedMatrix}:
-                value = value._get_value()
+                value = self._expect_type(
+                    value,
+                    (Matrix, TransposedMatrix),
+                    within=method_name,
+                )
             if rowsize is None or colsize is None:
                 if rowsize is None and colsize is None:
                     # C[i, j] << A  (mask doesn't matter)
@@ -953,7 +961,7 @@ class Matrix(BaseType):
                         # C[i, J](m) << c
                         # SS, SuiteSparse-specific: subassign
                         cfunc_name = "GrB_Row_subassign"
-                        value_vector = Vector.new(value.dtype, size=mask.mask.size, name="v_temp")
+                        value_vector = Vector.new(value.dtype, size=mask.mask._size, name="v_temp")
                         expr_repr = "[{1}, [{3} cols]](%s) << {0.name}" % mask.name
                     else:
                         # C(m)[i, J] << c
@@ -980,7 +988,7 @@ class Matrix(BaseType):
                         # C[I, j](m) << c
                         # SS, SuiteSparse-specific: subassign
                         cfunc_name = "GrB_Col_subassign"
-                        value_vector = Vector.new(value.dtype, size=mask.mask.size, name="v_temp")
+                        value_vector = Vector.new(value.dtype, size=mask.mask._size, name="v_temp")
                     else:
                         # C(m)[I, j] << c
                         # C[I, j] << c
@@ -1206,10 +1214,16 @@ class MatrixExpression(BaseExpression):
     # These raise exceptions
     __array__ = wrapdoc(Matrix.__array__)(Matrix.__array__)
     __bool__ = wrapdoc(Matrix.__bool__)(Matrix.__bool__)
-    __eq__ = wrapdoc(Matrix.__eq__)(Matrix.__eq__)
     __iand__ = wrapdoc(Matrix.__iand__)(Matrix.__iand__)
     __imatmul__ = wrapdoc(Matrix.__imatmul__)(Matrix.__imatmul__)
     __ior__ = wrapdoc(Matrix.__ior__)(Matrix.__ior__)
+    __iadd__ = _automethods.__iadd__
+    __ifloordiv__ = _automethods.__ifloordiv__
+    __imod__ = _automethods.__imod__
+    __imul__ = _automethods.__imul__
+    __ipow__ = _automethods.__ipow__
+    __isub__ = _automethods.__isub__
+    __itruediv__ = _automethods.__itruediv__
 
 
 class TransposedMatrix:
@@ -1302,6 +1316,15 @@ class TransposedMatrix:
     __rmatmul__ = Matrix.__rmatmul__
     __imatmul__ = Matrix.__imatmul__
 
+    # Bad sugar
+    __iadd__ = _automethods.__iadd__
+    __ifloordiv__ = _automethods.__ifloordiv__
+    __imod__ = _automethods.__imod__
+    __imul__ = _automethods.__imul__
+    __ipow__ = _automethods.__ipow__
+    __isub__ = _automethods.__isub__
+    __itruediv__ = _automethods.__itruediv__
+
     # Misc.
     isequal = Matrix.isequal
     isclose = Matrix.isclose
