@@ -17,6 +17,8 @@ from grblas.exceptions import (
     OutputNotEmpty,
 )
 
+from .conftest import autocompute
+
 
 @pytest.fixture
 def A():
@@ -131,7 +133,7 @@ def test_from_values_scalar():
     assert C.nvals == 3
     assert C.dtype == dtypes.INT64
     assert C.ss.is_iso
-    assert C.reduce_scalar(monoid.any) == 7
+    assert C.reduce_scalar(monoid.any).new() == 7
 
     # iso drumps duplicates
     C = Matrix.from_values([0, 1, 3, 0], [1, 1, 2, 1], 7)
@@ -140,7 +142,7 @@ def test_from_values_scalar():
     assert C.nvals == 3
     assert C.dtype == dtypes.INT64
     assert C.ss.is_iso
-    assert C.reduce_scalar(monoid.any) == 7
+    assert C.reduce_scalar(monoid.any).new() == 7
     with pytest.raises(ValueError, match="dup_op must be None"):
         Matrix.from_values([0, 1, 3, 0], [1, 1, 2, 1], 7, dup_op=binary.plus)
 
@@ -212,6 +214,8 @@ def test_build_scalar(A):
     A.clear()
     with pytest.raises(ValueError, match="lengths must match"):
         A.ss.build_scalar([0, 6], [0, 1, 2], 1)
+    with pytest.raises(InvalidValue):
+        A.ss.build_scalar([0, 5], [0, 1], None)
 
 
 def test_extract_values(A):
@@ -581,7 +585,7 @@ def test_assign(A):
     nvals = C.nvals
     C(C.S) << 1
     assert C.nvals == nvals
-    assert C.reduce_scalar().value == nvals
+    assert C.reduce_scalar().new() == nvals
     with pytest.raises(TypeError, match="Invalid type for index"):
         C[C, [1]] = C
 
@@ -1124,17 +1128,17 @@ def test_reduce_agg(A):
     assert w15.isequal(expected)
     assert w16.isequal(expected)
 
-    assert A.reduce_scalar(agg.sum).value == 47
-    assert A.reduce_scalar(agg.prod).value == 1270080
-    assert A.reduce_scalar(agg.count).value == 12
-    assert A.reduce_scalar(agg.count_nonzero).value == 12
-    assert A.reduce_scalar(agg.count_zero).value == 0
-    assert A.reduce_scalar(agg.sum_of_squares).value == 245
+    assert A.reduce_scalar(agg.sum).new() == 47
+    assert A.reduce_scalar(agg.prod).new() == 1270080
+    assert A.reduce_scalar(agg.count).new() == 12
+    assert A.reduce_scalar(agg.count_nonzero).new() == 12
+    assert A.reduce_scalar(agg.count_zero).new() == 0
+    assert A.reduce_scalar(agg.sum_of_squares).new() == 245
     assert A.reduce_scalar(agg.hypot).new().isclose(245 ** 0.5)
     assert A.reduce_scalar(agg.logaddexp).new().isclose(8.6071076)
     assert A.reduce_scalar(agg.logaddexp2).new().isclose(9.2288187)
     assert A.reduce_scalar(agg.mean).new().isclose(47 / 12)
-    assert A.reduce_scalar(agg.exists) == 1
+    assert A.reduce_scalar(agg.exists).new() == 1
 
     silly = agg.Aggregator(
         "silly",
@@ -1237,8 +1241,8 @@ def test_reduce_agg_firstlast(A):
     w6 = A.reduce_scalar(agg.last).new()
     assert w6 == 3
     B = Matrix.new(float, nrows=2, ncols=3)
-    assert B.reduce_scalar(agg.first).value is None
-    assert B.reduce_scalar(agg.last).value is None
+    assert B.reduce_scalar(agg.first).new().is_empty
+    assert B.reduce_scalar(agg.last).new().is_empty
     w7 = B.reduce_rows(agg.first).new()
     assert w7.isequal(Vector.new(float, size=B.nrows))
     w8 = B.reduce_columns(agg.last).new()
@@ -1349,7 +1353,7 @@ def test_reduce_column(A):
 def test_reduce_scalar(A):
     s = A.reduce_scalar(monoid.plus).new()
     assert s == 47
-    assert A.reduce_scalar(binary.plus).value == 47
+    assert A.reduce_scalar(binary.plus).new() == 47
     with pytest.raises(TypeError, match="Expected type: Monoid"):
         A.reduce_scalar(binary.minus)
 
@@ -1366,7 +1370,7 @@ def test_reduce_scalar(A):
     assert t == 47.0
     t(accum=binary.times) << A.reduce_scalar(monoid.plus)
     assert t == 47 * 47
-    assert A.reduce_scalar(monoid.plus[dtypes.UINT64]).value == 47
+    assert A.reduce_scalar(monoid.plus[dtypes.UINT64]).new() == 47
     # Make sure we accumulate as a float, not int
     t.value = 1.23
     t(accum=binary.plus) << A.reduce_scalar()
@@ -1557,9 +1561,8 @@ def test_bad_init():
         Matrix(None, float, name="bad_matrix")
 
 
-def test_no_equals(A):
-    with pytest.raises(TypeError, match="not defined for objects of type"):
-        A == A
+def test_equals(A):
+    assert (A == A).new().reduce_scalar(monoid.land).new()
 
 
 def test_bad_update(A):
@@ -1610,11 +1613,9 @@ def test_import_export(A, do_iso, methods):
         d = getattr(A1.ss, out_method)("csr", give_ownership=True)
     else:
         d = getattr(A1.ss, out_method)("csr")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert d["nrows"] == 7
     assert d["ncols"] == 7
     assert_array_equal(d["indptr"], [0, 2, 4, 5, 7, 8, 9, 12])
@@ -1632,11 +1633,9 @@ def test_import_export(A, do_iso, methods):
 
     A2 = A.dup()
     d = getattr(A2.ss, out_method)("csc")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert d["nrows"] == 7
     assert d["ncols"] == 7
     assert_array_equal(d["indptr"], [0, 1, 2, 5, 7, 9, 11, 12])
@@ -1654,11 +1653,9 @@ def test_import_export(A, do_iso, methods):
 
     A3 = A.dup()
     d = getattr(A3.ss, out_method)("hypercsr")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert d["nrows"] == 7
     assert d["ncols"] == 7
     assert_array_equal(d["rows"], [0, 1, 2, 3, 4, 5, 6])
@@ -1677,11 +1674,9 @@ def test_import_export(A, do_iso, methods):
 
     A4 = A.dup()
     d = getattr(A4.ss, out_method)("hypercsc")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert d["nrows"] == 7
     assert d["ncols"] == 7
     assert_array_equal(d["cols"], [0, 1, 2, 3, 4, 5, 6])
@@ -1700,11 +1695,9 @@ def test_import_export(A, do_iso, methods):
 
     A5 = A.dup()
     d = getattr(A5.ss, out_method)("bitmapr")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert "nrows" not in d
     assert "ncols" not in d
     if not do_iso:
@@ -1754,11 +1747,9 @@ def test_import_export(A, do_iso, methods):
 
     A6 = A.dup()
     d = getattr(A6.ss, out_method)("bitmapc")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert_array_equal(d["bitmap"], bitmap)
     if not do_iso:
         assert_array_equal(
@@ -1785,11 +1776,9 @@ def test_import_export(A, do_iso, methods):
 
     A7 = A.dup()
     d = getattr(A7.ss, out_method)()
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     if in_method == "import":
         B7 = Matrix.ss.import_any(**d)
         assert B7.isequal(A)
@@ -1801,11 +1790,9 @@ def test_import_export(A, do_iso, methods):
 
     A8 = A.dup()
     d = getattr(A8.ss, out_method)("bitmapr", raw=True)
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     del d["nrows"]
     del d["ncols"]
     if in_method == "import":
@@ -1818,17 +1805,14 @@ def test_import_export(A, do_iso, methods):
 
     A9 = A.dup()
     d = getattr(A9.ss, out_method)("coo", sort=True)
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     assert d["nrows"] == 7
     assert d["ncols"] == 7
     assert d["rows"].shape == (12,)
     assert d["cols"].shape == (12,)
-    assert d["sorted_rows"]
-    assert "sorted_cols" not in d
+    assert d["sorted_cols"]
     assert_array_equal(d["rows"], [0, 0, 1, 1, 2, 3, 3, 4, 5, 6, 6, 6])
     assert_array_equal(d["cols"], [1, 3, 4, 6, 5, 0, 2, 5, 2, 2, 3, 4])
 
@@ -1854,13 +1838,12 @@ def test_import_export(A, do_iso, methods):
         C(C.S) << 1
     C1 = C.dup()
     d = getattr(C1.ss, out_method)("fullr")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
         assert "nrows" in d
         assert "ncols" in d
     else:
-        assert "is_iso" not in d
         assert "nrows" not in d
         assert "ncols" not in d
     assert d["values"].flags.c_contiguous
@@ -1887,11 +1870,9 @@ def test_import_export(A, do_iso, methods):
 
     C2 = C.dup()
     d = getattr(C2.ss, out_method)("fullc")
+    assert d["is_iso"] is do_iso
     if do_iso:
-        assert d["is_iso"] is True
         assert_array_equal(d["values"], [1])
-    else:
-        assert "is_iso" not in d
     if not do_iso:
         assert_array_equal(d["values"], [[1, 2], [3, 4]])
     assert d["values"].flags.f_contiguous
@@ -1928,6 +1909,33 @@ def test_import_export(A, do_iso, methods):
     ]:
         with pytest.raises(TypeError):
             Matrix.ss.import_any(nrows=3, ncols=3, values=a, **dict.fromkeys(bad_combos, a))
+    with pytest.raises(ValueError, match="Invalid format"):
+        A.ss.export("coobad")
+    D = Matrix.ss.import_csc(**A.ss.export("csc"))
+    info = D.ss.export("coo", sort=True)
+    assert info["sorted_rows"]
+    E = Matrix.ss.import_any(**info)
+    assert E.isequal(A)
+
+    info = D.ss.export("coor")
+    info["sorted_rows"] = False
+    with pytest.raises(ValueError, match="sorted_rows must be True"):
+        Matrix.ss.import_coor(**info)
+    info["sorted_rows"] = True
+    info["sorted_cols"] = False
+    del info["format"]
+    E = Matrix.ss.import_any(**info)
+    assert E.isequal(D)
+
+    info = D.ss.export("cooc")
+    info["sorted_cols"] = False
+    with pytest.raises(ValueError, match="sorted_cols must be True"):
+        Matrix.ss.import_cooc(**info)
+    info["sorted_cols"] = True
+    info["sorted_rows"] = False
+    del info["format"]
+    E = Matrix.ss.import_any(**info)
+    assert E.isequal(D)
 
 
 def test_import_on_view():
@@ -2040,7 +2048,17 @@ def test_import_export_auto(A, do_iso, methods):
         A(A.S) << 1
     A_orig = A.dup()
     out_method, in_method = methods
-    for format in ["csr", "csc", "hypercsr", "hypercsc", "bitmapr", "bitmapc", "coo"]:
+    for format in [
+        "csr",
+        "csc",
+        "hypercsr",
+        "hypercsc",
+        "bitmapr",
+        "bitmapc",
+        "coo",
+        "coor",
+        "cooc",
+    ]:
         for (
             sort,
             raw,
@@ -2135,8 +2153,8 @@ def test_import_export_auto(A, do_iso, methods):
 def test_no_bool_or_eq(A):
     with pytest.raises(TypeError, match="not defined"):
         bool(A)
-    with pytest.raises(TypeError, match="not defined"):
-        A == A
+    # with pytest.raises(TypeError, match="not defined"):
+    A == A
     with pytest.raises(TypeError, match="not defined"):
         bool(A.S)
     with pytest.raises(TypeError, match="not defined"):
@@ -2144,7 +2162,7 @@ def test_no_bool_or_eq(A):
     expr = A.ewise_mult(A)
     with pytest.raises(TypeError, match="not defined"):
         bool(expr)
-    with pytest.raises(TypeError, match="not defined"):
+    with pytest.raises(TypeError, match="not enabled"):
         expr == expr
     assigner = A[1, 2]()
     with pytest.raises(TypeError, match="not defined"):
@@ -2158,6 +2176,7 @@ def test_no_bool_or_eq(A):
         updater == updater
 
 
+@autocompute
 def test_bool_eq_on_scalar_expressions(A):
     expr = A.reduce_scalar()
     assert expr == 47
@@ -2186,6 +2205,16 @@ def test_bool_eq_on_scalar_expressions(A):
         range(expr)
 
 
+def test_bool_eq_on_scalar_expressions_no_auto(A):
+    expr = A.reduce_scalar()
+    with pytest.raises(TypeError, match="autocompute"):
+        expr == 47
+    with pytest.raises(TypeError, match="autocompute"):
+        bool(expr)
+    with pytest.raises(TypeError, match="autocompute"):
+        int(expr)
+
+
 def test_contains(A):
     assert (0, 1) in A
     assert (1, 0) in A.T
@@ -2369,6 +2398,7 @@ def test_nbytes(A):
     assert A.ss.nbytes > 0
 
 
+@autocompute
 def test_auto(A, v):
     expected = binary.times(A & A).new()
     for expr in [(A & A), binary.times(A & A)]:
@@ -2412,6 +2442,7 @@ def test_auto(A, v):
         assert expr.vxm(A).new(mask=expr.V).isequal(expected.vxm(A).new(mask=expected.V))
 
 
+@autocompute
 def test_auto_assign(A):
     expected = A.dup()
     B = A[1:4, 1:4].new()
@@ -2429,6 +2460,7 @@ def test_auto_assign(A):
     assert expected.isequal(A)
 
 
+@autocompute
 def test_expr_is_like_matrix(A):
     attrs = {attr for attr, val in inspect.getmembers(A)}
     expr_attrs = {attr for attr, val in inspect.getmembers(binary.times(A & A))}
@@ -2467,3 +2499,165 @@ def test_expr_is_like_matrix(A):
         "_prep_for_extract",
         "_extract_element",
     }
+
+
+def test_flatten(A):
+    data = [
+        [3, 0, 3, 5, 6, 0, 6, 1, 6, 2, 4, 1],
+        [0, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6],
+        [3, 2, 3, 1, 5, 3, 7, 8, 3, 1, 7, 4],
+    ]
+    # row-wise
+    indices = [row * A.ncols + col for row, col in zip(data[0], data[1])]
+    expected = Vector.from_values(indices, data[2], size=A.nrows * A.ncols)
+    for fmt in ["csr", "hypercsr", "bitmapr"]:
+        B = Matrix.ss.import_any(**A.ss.export(format=fmt))
+        v = B.ss.flatten()
+        assert v.isequal(expected)
+        C = v.ss.reshape(*B.shape)
+        assert C.isequal(B)
+    B(mask=~B.S)[:, :] = 10
+    expected(mask=~expected.S)[:] = 10
+    B = Matrix.ss.import_fullr(**B.ss.export(format="fullr"))
+    v = B.ss.flatten()
+    assert v.isequal(expected)
+    C = v.ss.reshape(*B.shape)
+    assert C.isequal(B)
+    C = v.ss.reshape(B.shape)
+    assert C.isequal(B)
+
+    # column-wise
+    indices = [col * A.nrows + row for row, col in zip(data[0], data[1])]
+    expected = Vector.from_values(indices, data[2], size=A.nrows * A.ncols)
+    for fmt in ["csc", "hypercsc", "bitmapc"]:
+        B = Matrix.ss.import_any(**A.ss.export(format=fmt))
+        v = B.ss.flatten(order="col")
+        assert v.isequal(expected)
+        C = v.ss.reshape(*B.shape, order="col")
+        assert C.isequal(B)
+    B(mask=~B.S)[:, :] = 10
+    expected(mask=~expected.S)[:] = 10
+    B = Matrix.ss.import_fullc(**B.ss.export(format="fullc"))
+    v = B.ss.flatten(order="F")
+    assert v.isequal(expected)
+    C = v.ss.reshape(*B.shape, order="F")
+    assert C.isequal(B)
+    C = v.ss.reshape(B.shape, order="F")
+    assert C.isequal(B)
+    with pytest.raises(ValueError, match="Bad value for order"):
+        A.ss.flatten(order="bad")
+    with pytest.raises(ValueError, match="cannot reshape"):
+        v.ss.reshape(100, 100)
+    with pytest.raises(ValueError):
+        v.ss.reshape(A.shape + (1,))
+
+
+def test_autocompute_argument_messages(A, v):
+    with pytest.raises(TypeError, match="autocompute"):
+        A.ewise_mult(A & A)
+    with pytest.raises(TypeError, match="autocompute"):
+        A.mxv(A @ v)
+
+
+@autocompute
+def test_infix_sugar(A):
+    assert type(A + 1) is not Matrix
+    assert binary.plus(A, 1).isequal(A + 1)
+    assert binary.plus(A.T, 1).isequal(A.T + 1)
+    assert binary.plus(1, A).isequal(1 + A)
+    assert binary.minus(A, 1).isequal(A - 1)
+    assert binary.minus(1, A).isequal(1 - A)
+    assert binary.times(A, 2).isequal(A * 2)
+    assert binary.times(2, A).isequal(2 * A)
+    assert binary.truediv(A, 2).isequal(A / 2)
+    assert binary.truediv(5, A).isequal(5 / A)
+    assert binary.floordiv(A, 2).isequal(A // 2)
+    assert binary.floordiv(5, A).isequal(5 // A)
+    assert binary.numpy.mod(A, 2).isequal(A % 2)
+    assert binary.numpy.mod(5, A).isequal(5 % A)
+    assert binary.pow(A, 2).isequal(A ** 2)
+    assert binary.pow(2, A).isequal(2 ** A)
+    assert binary.pow(A, 2).isequal(pow(A, 2))
+    assert unary.ainv(A).isequal(-A)
+    assert unary.ainv(A.T).isequal(-A.T)
+    assert unary.lnot(A).isequal(~A)
+    assert unary.lnot(A.T).isequal(~A.T)
+    assert binary.lt(A, 4).isequal(A < 4)
+    assert binary.le(A, 4).isequal(A <= 4)
+    assert binary.gt(A, 4).isequal(A > 4)
+    assert binary.ge(A, 4).isequal(A >= 4)
+    assert binary.eq(A, 4).isequal(A == 4)
+    assert binary.ne(A, 4).isequal(A != 4)
+    x, y = divmod(A, 3)
+    assert binary.floordiv(A, 3).isequal(x)
+    assert binary.numpy.mod(A, 3).isequal(y)
+    assert binary.fmod(A, 3).isequal(y)
+    assert A.isequal(binary.plus((3 * x) & y))
+    x, y = divmod(-A, 3)
+    assert binary.floordiv(-A, 3).isequal(x)
+    assert binary.numpy.mod(-A, 3).isequal(y)
+    # assert binary.fmod(-A, 3).isequal(y)  # The reason we use numpy.mod
+    assert (-A).isequal(binary.plus((3 * x) & y))
+    x, y = divmod(3, A)
+    assert binary.floordiv(3, A).isequal(x)
+    assert binary.numpy.mod(3, A).isequal(y)
+    assert binary.fmod(3, A).isequal(y)
+    assert binary.plus(binary.times(A & x) & y).isequal(3 * unary.one(A))
+    x, y = divmod(-3, A)
+    assert binary.floordiv(-3, A).isequal(x)
+    assert binary.numpy.mod(-3, A).isequal(y)
+    # assert binary.fmod(-3, A).isequal(y)  # The reason we use numpy.mod
+    assert binary.plus(binary.times(A & x) & y).isequal(-3 * unary.one(A))
+
+    assert binary.eq(A & A).isequal(A == A)
+    assert binary.ne(A.T & A.T).isequal(A.T != A.T)
+    assert binary.lt(A & A.T).isequal(A < A.T)
+    assert binary.ge(A.T & A).isequal(A.T >= A)
+
+    B = A.dup()
+    B += 1
+    assert type(B) is Matrix
+    assert binary.plus(A, 1).isequal(B)
+    B = A.dup()
+    B -= 1
+    assert type(B) is Matrix
+    assert binary.minus(A, 1).isequal(B)
+    B = A.dup()
+    B *= 2
+    assert type(B) is Matrix
+    assert binary.times(A, 2).isequal(B)
+    B = A.dup(dtype=float)
+    B /= 2
+    assert type(B) is Matrix
+    assert binary.truediv(A, 2).isequal(B)
+    B = A.dup()
+    B //= 2
+    assert type(B) is Matrix
+    assert binary.floordiv(A, 2).isequal(B)
+    B = A.dup()
+    B %= 2
+    assert type(B) is Matrix
+    assert binary.numpy.mod(A, 2).isequal(B)
+    B = A.dup()
+    B **= 2
+    assert type(B) is Matrix
+    assert binary.pow(A, 2).isequal(B)
+
+    expr = binary.plus(A & A)
+    assert unary.abs(expr).isequal(abs(expr))
+    assert unary.ainv(expr).isequal(-expr)
+    assert unary.lnot(expr).isequal(~expr)
+    with pytest.raises(TypeError):
+        expr += 1
+    with pytest.raises(TypeError):
+        expr -= 1
+    with pytest.raises(TypeError):
+        expr *= 1
+    with pytest.raises(TypeError):
+        expr /= 1
+    with pytest.raises(TypeError):
+        expr //= 1
+    with pytest.raises(TypeError):
+        expr %= 1
+    with pytest.raises(TypeError):
+        expr **= 1
