@@ -129,7 +129,7 @@ class Vector(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce(monoid.land).value
+        return matches.reduce(monoid.land).new().value
 
     def isclose(self, other, *, rel_tol=1e-7, abs_tol=0.0, check_dtype=False):
         """
@@ -153,7 +153,7 @@ class Vector(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce(monoid.land).value
+        return matches.reduce(monoid.land).new().value
 
     @property
     def size(self):
@@ -627,7 +627,11 @@ class Vector(BaseType):
 
         if output_type(value) is Vector:
             if type(value) is not Vector:
-                value = value._get_value()
+                value = self._expect_type(
+                    value,
+                    Vector,
+                    within=method_name,
+                )
             if is_submask:
                 if isize is None:
                     # v[i](m) << w
@@ -816,10 +820,16 @@ class VectorExpression(BaseExpression):
     # These raise exceptions
     __array__ = wrapdoc(Vector.__array__)(Vector.__array__)
     __bool__ = wrapdoc(Vector.__bool__)(Vector.__bool__)
-    __eq__ = wrapdoc(Vector.__eq__)(Vector.__eq__)
     __iand__ = wrapdoc(Vector.__iand__)(Vector.__iand__)
     __imatmul__ = wrapdoc(Vector.__imatmul__)(Vector.__imatmul__)
     __ior__ = wrapdoc(Vector.__ior__)(Vector.__ior__)
+    __iadd__ = _automethods.__iadd__
+    __ifloordiv__ = _automethods.__ifloordiv__
+    __imod__ = _automethods.__imod__
+    __imul__ = _automethods.__imul__
+    __ipow__ = _automethods.__ipow__
+    __isub__ = _automethods.__isub__
+    __itruediv__ = _automethods.__itruediv__
 
 
 class _VectorAsMatrix:
